---
grand_parent: 32-bit MPU Reference Applications
parent: SAMA5D29 Curiosity Development Board
title: Getting started on CAN Protocol (CAN-FD) SAMA5D29 Curiosity Development Board
---

<img src = "images/microchip_logo.png">
<img src = "images/microchip_mplab_harmony_logo_small.png">

# Getting started on CAN Protocol (CAN-FD) SAMA5D29 Curiosity Development Board

## Description:
> This application transmits and receives CAN FD messages from MCAN0 to MCAN1 or vice versa. To run this application on SAMA5D29 Curiosity Development Board, two mikroBUS ATA6563 click boards (MCAN0 & MCAN1) are required. MCAN0 and MCAN1 acts as different nodes on the CAN bus. User can send and receive CAN-FD messages from/to respective CAN node through UART console on the PC. <br>

- USER input through UART console:

	| User input | Functions |
	| ------ | ------- |
	| Press `0` on the UART console | CAN-FD standard message transmit From MCAN0 to MCAN1 |
	| Press `1` on the UART console | CAN-FD standard message transmit From MCAN0 to MCAN1 |
	| Press `2` on the UART console | CAN-FD Extended message transmit From MCAN0 to MCAN1 |
	| Press `3` on the UART console | CAN-FD Extended message transmit From MCAN0 to MCAN1 |
	| Press `4` on the UART console | CAN-FD standard message transmit From MCAN1 to MCAN0 |
	| Press `5` on the UART console | CAN-FD standard message transmit From MCAN1 to MCAN0 |
	| Press `6` on the UART console | CAN-FD Extended message transmit From MCAN1 to MCAN0 |
	| Press `7` on the UART console | CAN-FD Extended message transmit From MCAN1 to MCAN0 |
	| Press `M` or `m` on the UART console | Display menu |


## Key Highlights of [SAMA5D29 Curiosity Development Board](https://www.microchip.com/en-us/development-tool/EV07R15A):


* External Non-Volatile Memories like QSPI, SD, and micro-SD card interfaces.
* Additional sensors can be interfaced using "click boards" through an on-board [mikroBUS connector](https://www.mikroe.com/click).
* Two mechanical programmable buttons.
* One User Input Switch and one RGB LED.
* UART, USB, CAN, Ethernet, Audio and JTAG.
* LCD & ISC camera.
* Raspberry pi connectors.
<img src = "images/sam_a5d29_cu_top.png">
<img src = "images/sam_a5d29_cu_bottom.png">

## Modules/Technology Used:
- Peripheral Modules
    - UART0
    - MCAN0  
	- MCAN1
   
<img src = "images/project_graph.png" align="middle">

## Hardware Used:
- [SAMA5D29 Curiosity development board](https://www.microchip.com/en-us/development-tool/EV07R15A)
- [ATA6563 MikroBUS click board]( https://www.mikroe.com/ata6563-click )
- Micro-SD Card
	
##### Hardware Modifications to enable MCAN communications:
	
-  Enable MCAN0:
<img src = "images/can0_Connections1.png" align="middle">

<img src = "images/can0_Connections2.png" align="middle">

		Note: Follow the above steps to enable the MCAN1 communication.

-  Enable MCAN1:

	- Step1: <br>
	To disable MBUS_2_UART_TX & MBUS_2_UART_RX, remove the resistors from R192 & R193. <br>
	
	- Step2: <br>
		To enable MCAN1 communication, add 0R resistors across R188 & R189. <br>
		
	- Step3: <br>
		To disconnect LCD_D22 line, remove the R132 resistor. <br>
		
	- Step4: <br>
		To enable MCAN1 communication, add 22R resistor to R134. <br>
		
	- Step5: <br>
		To disconnect LCD_D23 line, remove the R136 resistor. <br>
		
	- Step6: <br>
		To enable MCAN1 communication, add 22R resistor to R138. <br>
		
	
	 Note: Follow the above steps to enable the MCAN1 communication. For more details, refer the [Hardware Modification to enable MCAN0 communication section](#hardware-modifications-to-enable-mcan-communications). 
	
	
##### Modified SAMA5D29 Curiosity Development Board : <br>
		
- Top-side: <br>
		
<img src = "images/hardware_changes_1.png" align="middle">  <br>
		
- Bottom-side: <br>

<img src = "images/hardware_changes_2.png" align="middle"> <br>

	 Note: The location of the resistors on the board is shown in the above picture.


## Software/Tools Used:
- [MPLAB® X IDE](https://microchipdeveloper.com/mplabx:installation)
- [MPLAB® XC32 Compiler](https://microchipdeveloper.com/install:xc32)
- [MPLAB® Code Configurator (MCC)](https://microchipdeveloper.com/install:mcc)

<span style="color:blue"> This project has been verified to work with the following versions of software tools:</span>  

Refer [Project Manifest](./firmware/src/config/sam_a5d29_cu/harmony-manifest-success.yml) present in harmony-manifest-success.yml under the project folder *firmware/src/config/sam_a5d29_cu* to know the **MPLAB® X IDE**, **MCC**, **libraries** version. 

## Hardware Setup:
- Power up the board by connecting the powered USB type-C cable to the USB port J1 on the SAMA5D29 curiosity development board.
- Connect USB-TTL cable to J28 (UART DEBUG).
- Remove/Open the jumper from J10 to disable QSPI boot & J14 to disable ETHERNET.
- Connect external JTAG debugger at J27 (JTAG connector).  

	<img src = "images/hardware_setup2.png">
	
- Plug the [ATA6563 mikroBUS 1 click board](https://www.mikroe.com/ata6563-click) to J19 (mikroBUS 1) of the [SAMA5D29 Curiosity Development Board](https://www.microchip.com/en-us/development-tool/EV07R15A) & [ATA6563 mikroBUS 2 click board](https://www.mikroe.com/ata6563-click) to J22 (mikroBUS 2) of the [SAMA5D29 Curiosity Development Board](https://www.microchip.com/en-us/development-tool/EV07R15A) as per the Pin connections shown below.
	
	- MCAN0 (PC10 & PC11) lines are routed to mikroBUS 1 connector (J19 Connector) 

		| mikroBUS 1 interface (MCAN0) | ATA6563 click Board |
		| ------ | ------- |
		| CANTX0 (PC10)| RX (PIN 14) |
		| CANRX0 (PC11) | TX (PIN 13) |
		| VDD_3V3  | 3V3 (PIN 7) |
		| VDD_5V | 5V (PIN 10) |
		| GND | GND (PIN 9) |
		
	
	- MCAN1 (PC26 & PC27) lines are routed to mikroBUS 2 connector (J22 Connector) 

		| mikroBUS 2 interface (MCAN1) | ATA6563 click Board |
		| ------ | ------- |
		| CANTX1 (PC26)| RX (PIN 14) |
		| CANRX1 (PC27) | TX (PIN 13) |
		| VDD_3V3  | 3V3 (PIN 7) |
		| VDD_5V | 5V (PIN 10) |
		| GND | GND (PIN 9) |
		

<img src = "images/hardware_setup1.png">

- Connect the jumper wires from the ATA6563 mikroBUS click 1 (CANL & CANH) to the ATA6563 mikroBUS click 2 (CANL & CANH). <br>

<img src = "images/ata6563_Connections.png">


## Developing a CAN-FD demo application

<details>
  <summary> Details
</summary>   <br> 

- Launch MPLAB® X IDE - From the main menu, click on File, then New Project. Under `Projects` choose `32-bit MCC Harmony Project`, click Next - For `Framework Path` choose the location on your PC where you want to download the Harmony 3 framework - Click Next.  <br>

<img src = "images/step1.png"> <br>
<img src = "images/step2.png">  <br>

- Enter `Location` , `Folder` and `Name of the project`. Click Next.  <br>

<img src = "images/step3.png">  <br>

- In the `Configuration Settings` window, for `Name`  enter `default`, for `Device Family` select `ATSAM`, for `Target Device` select `ATSAMA5D29`. Click Finish.  <br>  

<img src = "images/step4.png">  <br>

This creates an empty project and set this project as `main project`. If there are other projects open in the project explorer window, set this project as main project by right clicking on the project, choose `Set as Main Project`.  <br>

<img src = "images/step5.png">  <br>

- After the project is created, MCC will be automatically launched. (To launch MCC manually, from main menu, click on `Tools` -> `Embedded` -> `MPLAB® Code Configurator` or click MCC button in the MPLAB® X IDE tool bar.) It will launch `Content Manger Wizard`. Then select MPLAB® Harmony.  <br>

<img src = "images/step6.png">  <br>

- In addition to the required packages(csp, dev_packs), download the optional packages bsp, core and then click Finish. Content download will take some time. Please wait till all the contents are downloaded.  <br>

<img src = "images/step7.png">  <br>

- Now a project graph will be displayed. From `Device Resource` add `Board Support Packages`  for `SAM A5D29 Curiosity BSP` to `Project Graph`.  <br>

<img src = "images/step8.png">  <br>

- From `Device Resource`, `Peripherals` -> `UART` , add `UART0` to the `Project Graph`. This helps to print the debug messages (console serial communication messages) via comm port. <br>

<img src = "images/step9.png">  <br>

- In the project graph, right click on `UART in UART0` and then add consumers `STDIO`.  <br>

<img src = "images/step10.png">  <br>


<img src = "images/step11.png">  <br>

- From the project graph window, click the `system` and in the `Configuration Options` -> `Clock (PMC)` -> `Generic Clock Menu` -> `Generic Clock for Peripheral UART0`. Enable the `Generic Clock for UART0`. Select the `MCK_CLK` clock. <br>

<img src = "images/step12.png">  <br>

- From the project graph window, click the `UART0` and in the `Configuration Options` -> `Select Clock Source - MCK` & `Baud Rate - 115200`. <br>

<img src = "images/step13.png">  <br>

- From `Device Resource`, `Peripherals` -> `MCAN` , add `MCAN0` to the `Project Graph`. <br>

<img src = "images/step14.png">  <br>

- From the project graph window, click the `system` and in the `Configuration Options` -> `Clock (PMC)` -> `Generic Clock Menu` -> `Generic Clock for Peripheral MCAN0`. Enable the `Generic Clock for MCAN0`. Select the `UPLL_CLK` clock and set `Generic Clock Division Ratio to 12`. <br>

<img src = "images/step15.png">  <br>

- From `Device Resource`, `Peripherals` -> `MCAN` , add `MCAN1` to the `Project Graph`. <br>

<img src = "images/step16.png">  <br>

- From the project graph window, click the `system` and in the `Configuration Options` -> `Clock (PMC)` -> `Generic Clock Menu` -> `Generic Clock for Peripheral MCAN1`. Enable the `Generic Clock for MCAN1`. Select the `UPLL_CLK` clock and set `Generic Clock Division Ratio to 12`. <br>

<img src = "images/step17.png">  <br>

- From the project graph window, click the `MCAN0` and in the `Configuration Options` -> select MCAN Operation Mode to `CAN-FD`, Enable the `Interrupt Mode`, `Normal Bit Timing`, `Data Bit Timing`, `Use RX FIFO 0`, `Use RX FIFO 1`, `Use Dedicated Rx Buffer`, `Use TX FIFO`, `Standard Filter`, `Extended Filter` & Enable the `Timestamp Enable`. <br>

<img src = "images/step18.png">  <br>

- From the project graph window, click the `MCAN1` and in the `Configuration Options` -> select MCAN Operation Mode to `CAN-FD`, Enable the `Interrupt Mode`, `Normal Bit Timing`, `Data Bit Timing`, `Use RX FIFO 0`, `Use RX FIFO 1`, `Use Dedicated Rx Buffer`, `Use TX FIFO`, `Standard Filter`, `Extended Filter` & Enable the `Timestamp Enable`. <br>

<img src = "images/step21.png">  <br>

- From the project graph window, click the plugins and select the `Pin Configuration`. Pin Setting window will open.  <br>

<img src = "images/step24.png">  <br>

- Ensure  all the pins are configured as below:  <br>

	| Pin ID | Function |
    | ------ | ------- |
	| PA7  | LED_AH |
    | PA8  | LED_AH  |
	| PA9  | LED_AH  |
    | PA17 | SWITCH_AL |
	| PB26 | UART0_URXD0 |
    | PC27 | UART0_UTXD1 |
	| PC10 | MCAN0_CANTX0 |
    | PC11 | MCAN0_CANRX0 |
    | PC26 | MCAN1_CANTX1 |
    | PC27 | MCAN1_CANRX1 |
    |||

<img src = "images/step25.png">  <br>

- From the project graph window, click the plugins and select the `AIC Configuration`. AIC window will open. Ensure MCAN0 & MCAN1 interrupt handlers are enabled.  <br>

<img src = "images/step26.png">  <br> 
<img src = "images/step27.png">  <br>

- Now Save all. Then click generate code. This will generate code for all the Device Resource that have been added in the project graph.  <br>

<img src = "images/step28.png">  <br>

- Up to this point in the project creation process, you used MPLAB Code Configurator(MCC) to configure and generate code to initialize the device (SAMA5D29) and initialize MACN0 & MCAN1. <br>
  Refer [initialization.c](./firmware/src/config/sam_a5d29_cu/initialization.c) to find all the device resource added in the project graph got initialized.  All that is left is for the user to write the application code in the app.c file.
  Documentation for each of the peripheral libraries or diver libraries can be accessed as follows:
  1.	Peripheral libraries APIs can be accessed as a HTML file (*.html) from the Harmony 3 Framework path. (/framework_path/csp/docs/index.html )
  2.    BSP libraries APIs can be found in bsp.h
  3.    MCAN APIs can be accessed as a HTML file (*.html) from the Harmony 3 Framework path. (/framework_path/csp/docs/GUID-FE4048B2-547B-4753-A6AD-63A3D937288F.html)

- APIs used for this application are as follows:
	* static void display_menu(void)
	* static void print_message(uint8_t numberOfMessage, MCAN_RX_BUFFER *rxBuf, uint8_t rxBufLen, uint8_t rxFifoBuf,uint8_t mcanID)
	* void MCAN0_MessageRAMConfigSet(uint8_t *msgRAMConfigBaseAddress)
	* void MCAN0_TxFifoCallbackRegister(MCAN_TX_FIFO_CALLBACK callback, uintptr_t contextHandle)
	* void MCAN0_RxFifoCallbackRegister(MCAN_RX_FIFO_NUM rxFifoNum, MCAN_RX_FIFO_CALLBACK callback, uintptr_t contextHandle)
	* void MCAN0_RxBuffersCallbackRegister(MCAN_TXRX_BUFFERS_CALLBACK callback, uintptr_t contextHandle)
	* bool MCAN0_MessageTransmitFifo(uint8_t numberOfMessage, MCAN_TX_BUFFER *txBuffer)
	* bool MCAN0_MessageReceiveFifo(MCAN_RX_FIFO_NUM rxFifoNum, uint8_t numberOfMessage, MCAN_RX_BUFFER *rxBuffer)
	* bool MCAN0_MessageReceive(uint8_t bufferNumber, MCAN_RX_BUFFER *rxBuffer)
	* LED_GREEN_On().
	* LED_GREEN_Off().
	* LED_RED_On().
	* LED_RED_Off().

- [Sample code can be found here](./firmware/src/main.c). <br>

- Now right click on the project and click `Properties`. For `Connected Hardware Tool` -> select connected hardware debugger used, for `Compiler Toolchain` -> select XC32 and click Apply.  <br>

<img src = "images/step29.png">  <br>

- From J-32/J-Link, `Option categories` choose `Communication` and for `JTAG Method`, select `4-wire JTAG`.  <br>

<img src = "images/step30.png">  <br>

- Select Bootstrap for Categories and select the `Use Bootstrap` checkbox. For `bootstrap file` -> select the at91bootstrap elf file from [here](./firmware/mcan_fd_operation_interrupt_timestamp_sam_a5d29_cu.X)- Click on Apply and OK.  <br>
	
	***${ProjectDir}/at91bootstrap.elf*** <br>
	
<img src = "images/step31.png">  <br>

- By default, MPLAB X only produces ELF and Hex format output files.
    To generate application output in binary format, a post build step needs to be added to the project properties.
	To do this project properties, building, click the check box `Execute this line after build`, then enter the below command. <br>

	***${MP_CC_DIR}/xc32-objcopy -O binary ${DISTDIR}/${PROJECTNAME}.${IMAGE_TYPE}.elf ${DISTDIR}/harmony.bin*** <br>

<img src = "images/step32.png" align="middle"> <br>

- Clean and build the project. You should see a message on the output console that the project was successfully built. This completes the development of the MCAN getting started application.  <br>

- The harmony.bin binaries will be available at ./firmware/mcan_fd_operation_interrupt_timestamp_sam_a5d29_cu.X/dist/default/production.
</details> 

## Programming/Debugging Application Project on MPLAB® X IDE:
- Open the project (sama5d29_curiosity_dev_board/mcan/firmware/mcan_fd_operation_interrupt_timestamp_sam_a5d29_cu.X) in MPLAB® X IDE. 
- Now right click on the project and click `Properties`. For `Connected Hardware Tool` -> select connected external hardware debugger, for `Compiler Toolchain` -> select XC32 and click Apply.  <br>

<img src = "images/step29.png">  <br>

- From J-Link , `Option categories` choose `Communication` and for `JTAG Method`, select `4-wire JTAG`.  <br>

<img src = "images/step30.png">  <br>

- Select Bootstrap for Categories and select the `Use Bootstrap` checkbox. For `bootstrap file`, select the at91bootstrap elf file from [here](./firmware/mcan_fd_operation_interrupt_timestamp_sam_a5d29_cu.X/)- Click on Apply and OK.  <br>

<img src = "images/step31.png">  <br>

- Clean and build the project. You should see a message on the output console that the project was successfully built. <br>
- Debug the code by clicking on the "Debug" button in MPLAB® X IDE tool bar.  <br>
- Run the application by clicking "run" button in MPLAB® X IDE tool bar. <br>
- Open the Terminal application (Ex.: Tera term) on the computer. <br>
- Connect to the DEBUG/J-link Virtual COM port and configure the serial settings as follows: <br>
				* ***Baud Rate : 115200*** <br>
				* ***Data : 8 Bits*** <br>
				* ***Parity : None*** <br>
				* ***Stop : 1 Bit*** <br>
				* ***Flow Control : None*** <br>
- In the console associated with MCAN0 & MCAN1, press "0" to transmit a CAN-FD standard message From MCAN0 to MCAN1. <br> 
- Transmitted message description and status will be displayed in the console window. <br>
- Received message will be displayed in the console window. <br>
- Follow the sequence below to send and receive different messages: <br>
		- Press '1' on the UART console to transmit a CAN-FD standard message From MCAN0 to MCAN1 <br>
		- Press '2' on the UART console to transmit a CAN-FD extended message From MCAN0 to MCAN1 <br>
		- Press '3' on the UART console to transmit a CAN-FD extended message From MCAN0 to MCAN1 <br>
		- Press '4' on the UART console to transmit a CAN-FD standard message From MCAN1 to MCAN0 <br>
		- Press '5' on the UART console to transmit a CAN-FD standard message From MCAN1 to MCAN0 <br>
		- Press '6' on the UART console to transmit a CAN-FD extended message From MCAN1 to MCAN0 <br>
		- Press '7' on the UART console to transmit a CAN-FD extended message From MCAN1 to MCAN0 <br>

If the steps are executed in this sequence, the final output in the consoles will be as below: <br>

<img src = "images/step35.png"> <br>
<img src = "images/step36.png"> <br>
<img src = "images/step37.png"> <br>
<img src = "images/step38.png"> <br>

## Running the pre-built harmony application from macro-SD Card:
The pre-built application bin file can be programmed by following the below steps:

### Steps to program the bin file on macro-SD card
- Take a micro-SD Card formatted with FAT32 file system.  
- Copy the boot.bin and harmony.bin files from this [location](./hex) to the micro-SD card using your PC.
- Insert the macro-SD card to J6 on the SAMA5D29 Curiosity development board.

### Steps to run the bin file from macro-SD card 
- Perform a reset by pressing RESET button on the SAMA5D29 Curiosity Development Board

<img src = "images/step39.png"> <br>
<img src = "images/step36.png"> <br>
<img src = "images/step37.png"> <br>
<img src = "images/step38.png"> <br>

## Comments:
- This application demo builds and works by following the instructions above in "Running the Demo" section- If you need to enhance/customize this application demo, you need to use the MPLAB® Harmony v3 Software framework- Refer links below to setup and build your applications using MPLAB® Harmony.
    - [How to Setup MPLAB® Harmony v3 Software Development Framework](https://ww1.microchip.com/downloads/en/DeviceDoc/How_to_Setup_MPLAB_%20Harmony_v3_Software_Development_Framework_DS90003232C.pdf)
	- [Getting Started with MPLAB® Harmony v3 Using MPLAB® Code Configurator](https://www.youtube.com/watch?v=KdhltTWaDp0)
    - [MPLAB® Code Configurator Content Manager for MPLAB® Harmony v3 Projects](https://www.youtube.com/watch?v=PRewTzrI3iE)
	- [SAMA5D2 Series Boot Process](https://microchipdeveloper.com/xwiki/bin/view/products/mcu-mpu/32bit-mpu/sama5d2-series-boot-process/)
## Revision:
- v1.0.0 - Released demo application
