/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes


uint8_t Mcan0MessageRAM[MCAN0_MESSAGE_RAM_CONFIG_SIZE] __attribute__((aligned (32))) __attribute__((__section__(".region_nocache")));

/* Standard identifier id[28:18]*/
#define WRITE_ID(id) (id << 18)
#define READ_ID(id)  (id >> 18)
#define MCAN0         0U


/* Application's state machine enum */
typedef enum
{
    APP_STATE_MCAN0_RECEIVE,
    APP_STATE_MCAN0_TRANSMIT,
    APP_STATE_MCAN0_IDLE,
    APP_STATE_MCAN0_XFER_SUCCESSFUL,
    APP_STATE_MCAN0_XFER_ERROR
} MCAN0_APP_STATES;


/* Variable to save Tx/Rx transfer status and context */
static uint32_t status = 0;
static uint32_t xferContext = 0;

static uint8_t rxFiFo0[MCAN0_RX_FIFO0_SIZE];
static uint8_t rxFiFo1[MCAN0_RX_FIFO1_SIZE];
static uint8_t rxBuffer[MCAN0_RX_BUFFER_SIZE];

uint8_t mcan0_gfx_data[4]; 
uint8_t mcanRxBuffFlag = False;

MCAN_RX_BUFFER *rxBufMCAN0;

// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************

/* Data length code to Message Length */
static uint8_t MCANDlcToLengthGet(uint8_t dlc)
{
    uint8_t msgLength[] = {0U, 1U, 2U, 3U, 4U, 5U, 6U, 7U, 8U, 12U, 16U, 20U, 24U, 32U, 48U, 64U};
    return msgLength[dlc];
}

/* Print Rx Message */
static void print_message(uint8_t numberOfMessage, MCAN_RX_BUFFER *rxBuf, uint8_t rxBufLen, uint8_t rxFifoBuf,uint8_t mcanID)
{
    uint8_t length = 0;
    uint8_t msgLength = 0;
    uint32_t id = 0;

    if (rxFifoBuf == 0)
        printf(" Rx FIFO0 :");
    else if (rxFifoBuf == 1)
        printf(" Rx FIFO1 :");
    else if (rxFifoBuf == 2)
        printf(" Rx Buffer :");

    for (uint8_t count = 0; count < numberOfMessage; count++)
    {
        /* Print message to Console */
        printf(" New Message Received to CAN%d\r\n", mcanID);
        id = rxBuf->xtd ? rxBuf->id : READ_ID(rxBuf->id);
        msgLength = MCANDlcToLengthGet(rxBuf->dlc);
        length = msgLength;
        printf(" Message - Timestamp : 0x%x ID : 0x%x Length : 0x%x ", (unsigned int)rxBuf->rxts, (unsigned int)id, (unsigned int)msgLength);
        printf("Message : ");
        while(length)
        {
            printf("0x%x ", rxBuf->data[msgLength - length--]);
        }
        printf("\r\n");
        rxBuf += rxBufLen;
    }
}

/* This function will be called by MCAN PLIB when Message received in Rx Buffer */
void APP_MCAN0_RxBufferCallback(uint8_t bufferNumber, uintptr_t context)
{
    xferContext = context;

    /* Check MCAN Status */
    status = MCAN0_ErrorGet();

    if (((status & MCAN_PSR_LEC_Msk) == MCAN_ERROR_NONE) || ((status & MCAN_PSR_LEC_Msk) == MCAN_ERROR_LEC_NO_CHANGE))
    {
        switch ((MCAN0_APP_STATES)context)
        {
            case APP_STATE_MCAN0_RECEIVE:
            {
                memset(rxBuffer, 0x00, MCAN0_RX_BUFFER_ELEMENT_SIZE);
                if (MCAN0_MessageReceive(bufferNumber, (MCAN_RX_BUFFER *)rxBuffer) == true)
                {
                    LED_BLUE_On();
                    print_message(1, (MCAN_RX_BUFFER *)rxBuffer, MCAN0_RX_BUFFER_ELEMENT_SIZE, 2, MCAN0);
                    rxBufMCAN0 = (MCAN_RX_BUFFER *)rxBuffer;
                    //rxBufMCAN0->data[0];
                    printf(" CAN0 Success\r\n");
                    mcan0_gfx_data[0] = rxBufMCAN0->data[0];
                    mcan0_gfx_data[1] = rxBufMCAN0->data[1];
                    mcan0_gfx_data[2] = rxBufMCAN0->data[2];
                    mcan0_gfx_data[3] = rxBufMCAN0->data[3];
                    mcanRxBuffFlag = True;
                    //printf(" \r\n Speed: 0x%x \r\n",rxBufMCAN0->data[0]);
                    //printf(" \r\n RPM: 0x%x \r\n",rxBufMCAN0->data[1]);
                    //printf(" \r\n Speed: 0x%x \r\n",rxBufMCAN0->data[2]);
                    //printf(" \r\n RPM: 0x%x \r\n",rxBufMCAN0->data[3]);
                    LED_BLUE_Off();
                }
                else
                {
                    printf(" CAN0 Error in received message\r\n");
                }
                break;
            }
            default:
                break;
        }
    }
    else
    {
        printf(" CAN0 Error in received message\r\n");
    }
}

/* This function will be called by MCAN PLIB when Message received in Rx FIFO0 */
void APP_MCAN0_RxFifo0Callback(uint8_t numberOfMessage, uintptr_t context)
{
    xferContext = context;

    /* Check MCAN Status */
    status = MCAN0_ErrorGet();

    if (((status & MCAN_PSR_LEC_Msk) == MCAN_ERROR_NONE) || ((status & MCAN_PSR_LEC_Msk) == MCAN_ERROR_LEC_NO_CHANGE))
    {
        switch ((MCAN0_APP_STATES)context)
        {
            case APP_STATE_MCAN0_RECEIVE:
            {
                memset(rxFiFo0, 0x00, (numberOfMessage * MCAN0_RX_FIFO0_ELEMENT_SIZE));
                if (MCAN0_MessageReceiveFifo(MCAN_RX_FIFO_0, numberOfMessage, (MCAN_RX_BUFFER *)rxFiFo0) == true)
                {
                    LED_BLUE_On();
                    print_message(numberOfMessage, (MCAN_RX_BUFFER *)rxFiFo0, MCAN0_RX_FIFO0_ELEMENT_SIZE, 0,MCAN0);
                    printf(" CAN0 Success\r\n");
                    LED_BLUE_Off();
                }
                else
                {
                    printf(" CAN0 Error in received message\r\n");
                }
                break;
            }
            default:
                break;
        }
    }
    else
    {
        printf(" CAN0 Error in received message\r\n");
    }
}

/* This function will be called by MCAN PLIB when Message received in Rx FIFO1 */
void APP_MCAN0_RxFifo1Callback(uint8_t numberOfMessage, uintptr_t context)
{
    xferContext = context;

    /* Check MCAN Status */
    status = MCAN0_ErrorGet();

    if (((status & MCAN_PSR_LEC_Msk) == MCAN_ERROR_NONE) || ((status & MCAN_PSR_LEC_Msk) == MCAN_ERROR_LEC_NO_CHANGE))
    {
        switch ((MCAN0_APP_STATES)context)
        {
            case APP_STATE_MCAN0_RECEIVE:
            {
                memset(rxFiFo1, 0x00, (numberOfMessage * MCAN0_RX_FIFO1_ELEMENT_SIZE));
                if (MCAN0_MessageReceiveFifo(MCAN_RX_FIFO_1, numberOfMessage, (MCAN_RX_BUFFER *)rxFiFo1) == true)
                {
                    LED_BLUE_On();
                    print_message(numberOfMessage, (MCAN_RX_BUFFER *)rxFiFo1, MCAN0_RX_FIFO1_ELEMENT_SIZE, 1,MCAN0);
                    printf(" CAN0 Success\r\n");
                    LED_BLUE_Off();
                }
                else
                {
                    printf(" CAN0 Error in received message\r\n");
                }
                break;
            }
            default:
                break;
        }
    }
    else
    {
        printf(" CAN0 Error in received message\r\n");
    }
}


int main ( void )
{
        
    /* Initialize all modules */
    SYS_Initialize ( NULL );

    MCAN0_MessageRAMConfigSet(Mcan0MessageRAM);
    
    MCAN0_RxFifoCallbackRegister(MCAN_RX_FIFO_0, APP_MCAN0_RxFifo0Callback, APP_STATE_MCAN0_RECEIVE);
    MCAN0_RxFifoCallbackRegister(MCAN_RX_FIFO_1, APP_MCAN0_RxFifo1Callback, APP_STATE_MCAN0_RECEIVE);
    MCAN0_RxBuffersCallbackRegister(APP_MCAN0_RxBufferCallback, APP_STATE_MCAN0_RECEIVE);
    
    while ( true )
    {
        /* Maintain state machines of all polled MPLAB Harmony modules. */
        SYS_Tasks ( );
    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

