---
grand_parent: 32-bit MPU Reference Applications
parent: SAMA5D29 Curiosity Development Board
title: Application Demo on CAN Protocol (CAN-FD) with LCD on SAMA5D29 Curiosity Development Board
---

<img src = "images/microchip_logo.png">

<img src = "images/microchip_mplab_harmony_logo_small.png">

# Application Demo on CAN Protocol (CAN-FD) with LCD on SAMA5D29 Curiosity Development Board



## Description:
> The demonstration of this application showcases a user interface for a vehicle dashboard, integrated with CAN-FD. The dashboard features a gauge that displays the engine's RPM (revolutions per minute) and the vehicle's speed. The metrics, which are acquired through CAN-FD, are graphically displayed on the Legato dashboard. Upon reception of data on CAN-FD, the Blue LED will flash. <br>


## Key Highlights of [SAMA5D29 Curiosity Development Board](https://www.microchip.com/en-us/development-tool/EV07R15A):


* External Non-Volatile Memories like QSPI, SD, and micro-SD card interfaces.
* Additional sensors can be interfaced using "click boards" through an on-board [mikroBUS connector](https://www.mikroe.com/click).
* Two mechanical programmable buttons.
* One User Input Switch and one RGB LED.
* UART, USB, CAN, Ethernet, Audio and JTAG.
* LCD & ISC camera.
* Raspberry pi connectors.


## Modules/Technology Used:
- Peripheral Modules
	- Maxtouch Controller Interface
	- LCDC display driver
	- TWIHS0
	- TC0
	- UART0
    - MCAN0  

<img src = "images/project_graph.png" align="middle">

## Hardware Used:
- [SAMA5D29 Curiosity development board](https://www.microchip.com/en-us/development-tool/EV07R15A)
- [High-Performance WVGA LCD Display Module with maXTouch ® Technology]( https://www.microchip.com/developmenttools/ProductDetails/AC320005-5#additional-summary )
- [ATA6563 mikroBUS click board]( https://www.mikroe.com/ata6563-click )
- Ribbon Cable - awm 20624 cable 80c 60v vw-1
- [PCAN-USB opto-decoupled]( https://www.peak-system.com/PCAN-USB.199.0.html?&L=1 ) - IPEH-002022
- [PCAN-CABLE-2](https://www.peak-system.com/PCAN-Cable-1-2.272.0.html?&L=1) - IPEK-003001
- Micro-SD Card
	

## Software/Tools Used:
- [MPLAB® X IDE](https://microchipdeveloper.com/mplabx:installation)
- [MPLAB® XC32 Compiler](https://microchipdeveloper.com/install:xc32)
- [MPLAB® Code Configurator (MCC)](https://microchipdeveloper.com/install:mcc)
- [PCAN-view](https://www.peak-system.com/PCAN-USB.199.0.html?&L=1)

<span style="color:blue"> This project has been verified to work with the following versions of software tools:</span>  

Refer [Project Manifest](./firmware/src/config/sam_a5d29_cu/harmony-manifest-success.yml) present in harmony-manifest-success.yml under the project folder *firmware/src/config/sam_a5d29_cu* to know the **MPLAB® X IDE**, **MCC**, **libraries** version. 

## Hardware Setup:

1. A modification to the hardware is necessary for the MCAN0 communication interface on the SAMA5D29 Curiosity Development Board. Follow the steps outlined in the [Hardware Modifications to enable MCAN communications](../SAMA5D29_CAN_FD/readme.md#hardware-modifications-to-enable-mcan-communications) section.
2. Plug the [ATA6563 mikroBUS click](https://www.mikroe.com/ata6563-click) 1 board to J19 (mikroBUS 1) of the [SAMA5D29 Curiosity Development Board](https://www.microchip.com/en-us/development-tool/EV07R15A). 
3. Connect the ribbon cable from the display to the J23 connector of the SAMA5D29 Curiosity Development Board.
4. Connect the PCAN Cable between the ATA6563 mikroBUS Click on D9 connector and PCAN USB on D9 connector.
5. Remove/Open the jumper from J10 to disable QSPI boot & J14 to disable ETHERNET.
6. Power up the board by connecting the powered USB type-C cable to the USB port J1 on the SAMA5D29 curiosity development board.
7. Connect USB-TTL cable to J28 (UART DEBUG).
8. Connect external JTAG debugger at J27 (JTAG connector).  

<img src = "images/hardware_setup_1.png">
 

## PCAN-View configuration:

<details>
  <summary> Details
</summary>   <br> 

- Download the [PCAN-view](https://www.peak-system.com/quick/DrvSetup) and install it in PC.

- Open PCAN-view then Connect window will appear.

- Navigate to the CAN Setup tab and configure it as illustrated in the below image.

<img src = "images/setup.png">

- Navigate  to the Acceptance filter tab and configure it as illustrated in the below image. Once done, click `OK` to complete the connection setup.

<img src = "images/setup2.png">

- Create a new message to transmit from PCAN. Click on the mail icon to compose new transmit message.

<img src = "images/setup3.png">

- Start the CAN-FD Communication.

<img src = "images/setup4.png">

</details> 


## Programming/Debugging Application Project on MPLAB® X IDE:
- Open the project (sama5d29_curiosity_dev_board/sama5d29_cu_legato_db_mcan/firmware/legato_db_mcan_sam_a5d29_cu_tm5000.X) in MPLAB® X IDE. 
- Now right click on the project and click `Properties`. For `Connected Hardware Tool` -> select connected external hardware debugger, for `Compiler Toolchain` -> select XC32 and click Apply.  <br>

<img src = "images/step5.png">  <br>

- From J-Link , `Option categories` choose `Communication` and for `JTAG Method`, select `4-wire JTAG`.  <br>

<img src = "images/step6.png">  <br>

- Select Bootstrap for Categories and select the `Use Bootstrap` checkbox. For `bootstrap file`, select the at91bootstrap elf file from [here](./firmware/legato_db_mcan_sam_a5d29_cu_tm5000.X/)- Click on Apply and OK.  <br>

<img src = "images/setup7.png">  <br>

- Clean and build the project. You should see a message on the output console that the project was successfully built. <br>
- Debug the code by clicking on the "Debug" button in MPLAB® X IDE tool bar.  <br>
- Run the application by clicking "run" button in MPLAB® X IDE tool bar. <br>
- Open the Terminal application (Ex.: Tera term) on the computer. <br>
- Connect to the DEBUG/J-link Virtual COM port and configure the serial settings as follows: <br>
				* ***Baud Rate : 115200*** <br>
				* ***Data : 8 Bits*** <br>
				* ***Parity : None*** <br>
				* ***Stop : 1 Bit*** <br>
				* ***Flow Control : None*** <br>

If the steps are executed in this sequence, the final output in the consoles and LCD will be as below: <br>

<img src = "images/step8.png"> <br>
<img src = "images/step11.png"> <br>
- Transmit the data from PCAN to MCAN0. Upon each receipt of CAN-FD data, the blue LED will intermittently blink to indicate successful reception. 
<img src = "images/step9.gif"> <br>

## Running the pre-built harmony application from macro-SD Card:
The pre-built application bin file can be programmed by following the below steps:

### Steps to program the bin file on macro-SD card
- Take a micro-SD Card formatted with FAT32 file system.  
- Copy the boot.bin and harmony.bin files from this [location](./hex) to the micro-SD card using your PC.
- Insert the macro-SD card to J6 on the SAMA5D29 Curiosity development board.

### Steps to run the bin file from macro-SD card 
- Perform a reset by pressing RESET button on the SAMA5D29 Curiosity Development Board
- Transmit the data from PCAN to MCAN0. Upon each receipt of CAN-FD data, the blue LED will intermittently blink and Gauge movement can be observed accordingly on the legato dashboard in the LCD display. 

<img src = "images/step9.gif"> <br>

## Comments:
- This application demo builds and works by following the instructions above in "Running the Demo" section- If you need to enhance/customize this application demo, you need to use the MPLAB® Harmony v3 Software framework- Refer links below to setup and build your applications using MPLAB® Harmony.
    - [How to Setup MPLAB® Harmony v3 Software Development Framework](https://ww1.microchip.com/downloads/en/DeviceDoc/How_to_Setup_MPLAB_%20Harmony_v3_Software_Development_Framework_DS90003232C.pdf)
	- [Getting Started with MPLAB® Harmony v3 Using MPLAB® Code Configurator](https://www.youtube.com/watch?v=KdhltTWaDp0)
    - [MPLAB® Code Configurator Content Manager for MPLAB® Harmony v3 Projects](https://www.youtube.com/watch?v=PRewTzrI3iE)
	- [SAMA5D2 Series Boot Process](https://microchipdeveloper.com/xwiki/bin/view/products/mcu-mpu/32bit-mpu/sama5d2-series-boot-process/)
	- [Microchip Graphics Composer](https://github.com/mchpgfx/legato.docs/wiki/UserGuide-LegatoComposer)
	- [Click Here for more graphics demos](https://github.com/Microchip-MPLAB-Harmony/gfx_apps_sam_9x60)  
	- [MPLAB® Harmony v3 Graphics Suite Wiki](https://github.com/mchpgfx/legato.docs/wiki)
	- [MPLAB® Harmony v3 Graphics Example Applications at MPLAB® Discover](https://mplab-discover.microchip.com/v2/category/com.microchip.code.examples/com.microchip.ide.project?dsl=Legato)
	- [Getting Started with Graphics on SAM9X60 Curiosity Development Board Video](https://www.youtube.com/watch?v=osuWUntdWWk)
	
## Revision:
- v1.0.0 - Released demo application
